@@ -40,6 +40,7 @@ from ironic_python_agent import errors
 from ironic_python_agent.extensions import base as ext_base
 from ironic_python_agent import netutils
 from ironic_python_agent import raid_utils
+from ironic_python_agent import tls_utils
 from ironic_python_agent import utils
 
 _global_managers = None
@@ -53,7 +54,9 @@ UNIT_CONVERTER.define('bytes = []')
 UNIT_CONVERTER.define('MB = 1048576 bytes')
 _MEMORY_ID_RE = re.compile(r'^memory(:\d+)?$')
 NODE = None
-
+API_CLIENT = None
+API_LOOKUP_TIMEOUT = None
+API_LOOKUP_INTERVAL = None
 SUPPORTED_SOFTWARE_RAID_LEVELS = frozenset(['0', '1', '1+0', '5', '6'])
 
 RAID_APPLY_CONFIGURATION_ARGSINFO = {
@@ -133,10 +136,34 @@ def _check_for_iscsi():
                     "Error: %s", e)
 
 
+def _get_md_uuid(raid_device):
+    """Get the md UUID of a Software RAID device.
+
+    :param raid_device: A Software RAID block device name.
+    :returns: A string containing the UUID of an md device.
+    """
+    try:
+        out, _ = utils.execute('mdadm', '--detail', raid_device,
+                               use_standard_locale=True)
+    except processutils.ProcessExecutionError as e:
+        msg = ('Could not get the details of %(dev)s: %(err)s' %
+               {'dev': raid_device, 'err': e})
+        LOG.warning(msg)
+        return
+
+    lines = out.splitlines()
+    # the first line contains the md device itself
+    for line in lines[1:]:
+        match = re.search(r'UUID : ([a-f0-9:]+)', line)
+        if match:
+            return match.group(1)
+
+
 def _get_component_devices(raid_device):
     """Get the component devices of a Software RAID device.
 
-    Examine an md device and return its constituent devices.
+    Get the UUID of the md device and scan all other devices
+    for the same md UUID.
 
     :param raid_device: A Software RAID block device name.
     :returns: A list of the component devices.
@@ -144,22 +171,35 @@ def _get_component_devices(raid_device):
     if not raid_device:
         return []
 
-    try:
-        out, _ = utils.execute('mdadm', '--detail', raid_device,
-                               use_standard_locale=True)
-    except processutils.ProcessExecutionError as e:
-        msg = ('Could not get component devices of %(dev)s: %(err)s' %
-               {'dev': raid_device, 'err': e})
-        LOG.warning(msg)
+    md_uuid = _get_md_uuid(raid_device)
+    if not md_uuid:
         return []
+    LOG.debug('%s has UUID %s', raid_device, md_uuid)
 
     component_devices = []
-    lines = out.splitlines()
-    # the first line contains the md device itself
-    for line in lines[1:]:
-        device = re.findall(r'/dev/\w+', line)
-        component_devices += device
+    block_devices = list_all_block_devices()
+    block_devices.extend(list_all_block_devices(block_type='part',
+                                                ignore_raid=True))
+    for bdev in block_devices:
+        try:
+            out, _ = utils.execute('mdadm', '--examine', bdev.name,
+                                   use_standard_locale=True)
+        except processutils.ProcessExecutionError as e:
+            if "No md superblock detected" in str(e):
+                # actually not a component device
+                LOG.debug('Not a component device %s', bdev.name)
+                continue
+            else:
+                LOG.warning("Failed to examine device %s: %s",
+                            bdev.name, e)
+                continue
+        lines = out.splitlines()
+        for line in lines:
+            if md_uuid in line:
+                component_devices.append(bdev.name)
 
+    LOG.info('Found component devices for %s:',
+             raid_device, component_devices)
     return component_devices
 
 
@@ -470,6 +510,40 @@ def list_all_block_devices(block_type='disk',
     return devices
 
 
+def save_api_client(client=None, timeout=None, interval=None):
+    """Preserves access to the API client for potential later re-use."""
+    global API_CLIENT, API_LOOKUP_TIMEOUT, API_LOOKUP_INTERVAL
+
+    if client and timeout and interval and not API_CLIENT:
+        API_CLIENT = client
+        API_LOOKUP_TIMEOUT = timeout
+        API_LOOKUP_INTERVAL = interval
+
+
+def update_cached_node():
+    """Attmepts to update the node cache via the API"""
+    cached_node = get_cached_node()
+    if API_CLIENT:
+        LOG.info('Agent is requesting to perform an explicit node cache '
+                 'update. This is to pickup any chanages in the cache '
+                 'before deployment.')
+        try:
+            if cached_node is None:
+                uuid = None
+            else:
+                uuid = cached_node['uuid']
+            content = API_CLIENT.lookup_node(
+                hardware_info=list_hardware_info(use_cache=True),
+                timeout=API_LOOKUP_TIMEOUT,
+                starting_interval=API_LOOKUP_INTERVAL,
+                node_uuid=uuid)
+            cache_node(content['node'])
+            return content['node']
+        except Exception as exc:
+            LOG.warning('Failed to update node cache. Error %s', exc)
+    return cached_node
+
+
 class HardwareSupport(object):
     """Example priorities for hardware managers.
 
@@ -597,7 +671,7 @@ class HardwareManager(object, metaclass=abc.ABCMeta):
     def get_memory(self):
         raise errors.IncompatibleHardwareMethodError
 
-    def get_os_install_device(self):
+    def get_os_install_device(self, permit_refresh=False):
         raise errors.IncompatibleHardwareMethodError
 
     def get_bmc_address(self):
@@ -612,6 +686,9 @@ class HardwareManager(object, metaclass=abc.ABCMeta):
     def get_interface_info(self, interface_name):
         raise errors.IncompatibleHardwareMethodError()
 
+    def generate_tls_certificate(self, ip_address):
+        raise errors.IncompatibleHardwareMethodError()
+
     def erase_block_device(self, node, block_device):
         """Attempt to erase a block device.
 
@@ -1052,13 +1129,18 @@ class GenericHardwareManager(HardwareManager):
             )
         return block_devices
 
-    def get_os_install_device(self):
+    def get_os_install_device(self, permit_refresh=False):
         cached_node = get_cached_node()
         root_device_hints = None
         if cached_node is not None:
             root_device_hints = (
                 cached_node['instance_info'].get('root_device')
                 or cached_node['properties'].get('root_device'))
+            if permit_refresh and not root_device_hints:
+                cached_node = update_cached_node()
+                root_device_hints = (
+                    cached_node['instance_info'].get('root_device')
+                    or cached_node['properties'].get('root_device'))
             LOG.debug('Looking for a device matching root hints %s',
                       root_device_hints)
 
@@ -1836,6 +1918,7 @@ class GenericHardwareManager(HardwareManager):
             raise errors.SoftwareRAIDError(msg)
 
     def _delete_config_pass(self, raid_devices):
+        all_holder_disks = []
         for raid_device in raid_devices:
             component_devices = _get_component_devices(raid_device.name)
             if not component_devices:
@@ -1888,14 +1971,11 @@ class GenericHardwareManager(HardwareManager):
                     LOG.warning('Failed to remove superblock from %s: %s',
                                 raid_device.name, e)
 
-            # Remove the partitions we created during create_configuration.
-            for holder_disk in holder_disks:
-                LOG.debug('Removing partitions on %s', holder_disk)
-                try:
-                    utils.execute('wipefs', '-af', holder_disk)
-                except processutils.ProcessExecutionError:
-                    LOG.warning('Failed to remove partitions on %s',
-                                holder_disk)
+            # NOTE(arne_wiebalck): We cannot delete the partitions right
+            # away since there may be other partitions on the same disks
+            # which are members of other RAID devices. So we remember them
+            # for later.
+            all_holder_disks.extend(holder_disks)
 
             LOG.info('Deleted Software RAID device %s', raid_device.name)
 
@@ -1941,6 +2021,17 @@ class GenericHardwareManager(HardwareManager):
                 LOG.warning('Failed to remove superblock from %s: %s',
                             raid_device.name, e)
 
+        # Erase all partition tables we created
+        all_holder_disks_uniq = list(
+            collections.OrderedDict.fromkeys(all_holder_disks))
+        for holder_disk in all_holder_disks_uniq:
+            LOG.info('Removing partitions on holder disk %s', holder_disk)
+            try:
+                utils.execute('wipefs', '-af', holder_disk)
+            except processutils.ProcessExecutionError as e:
+                LOG.warning('Failed to remove partitions on %s',
+                            holder_disk, e)
+
         LOG.debug("Finished deleting Software RAID(s)")
 
     def validate_configuration(self, raid_config, node):
@@ -2050,6 +2141,10 @@ class GenericHardwareManager(HardwareManager):
         # The result is asynchronous, wait here.
         cmd.join()
 
+    def generate_tls_certificate(self, ip_address):
+        """Generate a TLS certificate for the IP address."""
+        return tls_utils.generate_tls_certificate(ip_address)
+
 
 def _compare_extensions(ext1, ext2):
     mgr1 = ext1.obj
