@@ -26,6 +26,9 @@ from ironic_python_agent import utils
 
 
 class FakeAgent(object):
+
+    iscsi_started = False
+
     def get_node_uuid(self):
         return 'my_node_uuid'
 
@@ -48,8 +51,11 @@ class TestISCSIExtensionTgt(base.IronicAgentTest):
                                 mock_destroy):
         mock_dispatch.return_value = self.fake_dev
         mock_execute.return_value = ('', '')
+        self.assertFalse(self.agent_extension.agent.iscsi_started)
+
         result = self.agent_extension.start_iscsi_target(iqn=self.fake_iqn)
 
+        self.assertTrue(self.agent_extension.agent.iscsi_started)
         expected = [mock.call('tgtd'),
                     mock.call('tgtadm', '--lld', 'iscsi', '--mode',
                               'target', '--op', 'show', attempts=10),
@@ -63,7 +69,8 @@ class TestISCSIExtensionTgt(base.IronicAgentTest):
                               '--op', 'bind', '--tid', '1',
                               '--initiator-address', 'ALL')]
         mock_execute.assert_has_calls(expected)
-        mock_dispatch.assert_called_once_with('get_os_install_device')
+        mock_dispatch.assert_called_once_with('get_os_install_device',
+                                              permit_refresh=True)
         self.assertEqual({'iscsi_target_iqn': self.fake_iqn},
                          result.command_result)
         self.assertFalse(mock_destroy.called)
@@ -92,7 +99,8 @@ class TestISCSIExtensionTgt(base.IronicAgentTest):
                               '--op', 'bind', '--tid', '1',
                               '--initiator-address', 'ALL')]
         mock_execute.assert_has_calls(expected)
-        mock_dispatch.assert_called_once_with('get_os_install_device')
+        mock_dispatch.assert_called_once_with('get_os_install_device',
+                                              permit_refresh=True)
         self.assertEqual({'iscsi_target_iqn': self.fake_iqn},
                          result.command_result)
 
@@ -113,7 +121,8 @@ class TestISCSIExtensionTgt(base.IronicAgentTest):
                               '--op', 'show', attempts=10)]
 
         mock_execute.assert_has_calls(expected)
-        mock_dispatch.assert_called_once_with('get_os_install_device')
+        mock_dispatch.assert_called_once_with('get_os_install_device',
+                                              permit_refresh=True)
         self.assertFalse(mock_destroy.called)
 
     @mock.patch.object(iscsi, '_wait_for_tgtd', autospec=True)
@@ -132,7 +141,8 @@ class TestISCSIExtensionTgt(base.IronicAgentTest):
                               'target', '--op', 'new', '--tid', '1',
                               '--targetname', self.fake_iqn)]
         mock_execute.assert_has_calls(expected)
-        mock_dispatch.assert_called_once_with('get_os_install_device')
+        mock_dispatch.assert_called_once_with('get_os_install_device',
+                                              permit_refresh=True)
 
     def test_start_iscsi_target_fail_command_not_exist(self, mock_execute,
                                                        mock_dispatch,
